package controller;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.LinkedList;
import java.util.Map;

import javax.swing.JOptionPane;

import model.Comment;
import model.MorphoTags;
import model.StateAnnotation;
import model.WordNet;
import model.ontology.Node;
import model.ontology.Ontology;
import nlputil.Lemmar;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;

import view.AnnotersView;
import view.CommentView;
import view.LimitAnnotationView;
import view.SynsetView;
import view.TextView;
import view.Translates;
import view.components.SynsetCellRender;
import view.components.Text;
import view.components.Tokens;
import edu.smu.tspell.wordnet.Synset;

/**
 * Class that control the annotation
 * 
 * @author fernando
 *
 */
public class Annotation {
	private Translates t;
	private SynsetView sv;
	private TextView tv;
	private CommentView cv;
	private LimitAnnotationView lav;
	private Tokens word;
	public static boolean isUpdated = false; 
	
	//annotation process
	private String translate;
	private String name_file = "";
	private boolean manual_translate;
	
	//used to save the annotation state
	private int count;
	
	private AnnotersView av;

	/**
	 * Constructor
	 * @param t Translate UI
	 * @param sv Synset UI
	 * @param tv Text UI
	 * @param annoters Annoters UI
	 * @param cv Comment UI
	 * @param lav Limit of annotation UI
	 */
	public Annotation(Translates t, SynsetView sv, TextView tv , AnnotersView annoters, CommentView cv, LimitAnnotationView lav){
		this.t = t;
		this.sv = sv;
		this.tv = tv;
		this.count = 0;
		this.av = annoters;
		this.cv = cv;
		this.lav = lav;
		isUpdated = false; 
		inicialize( );
	}
		
	/**
	 * This method inicialize the values of annotation for a word 
	 */
	private void inicialize(){
		this.word = null;
		this.translate = null;
		this.manual_translate = false;
		sv.clear();
	}
	
	/**
	 * Method for translating a word and insert the translates possible
	 * in the translation list 
	 * 
	 * @param word Word to be translated
	 */
	public void translate( Tokens word ){
		
		inicialize();				
		this.word = word;
		
		
		if( !word.isAnnoted() ){
			String lemma = word.getLemma();
			t.translate( lemma , word.getLanguage() );
			
		}else{
			t.setTranslate( word.getTtranslateSet() );
			t.setSelectedTranslate( word.getTranslate() );
			translate = word.getTranslate();
			sv.search( word.getTranslate(), word.getMorphoTag().name());
			sv.setSelectedSynset( word.getSynset() );
		}
	}

	/**
	 * Method used to search synsets of a word and
	 * put them in the list of synsets possible
	 *  
	 * @param word key for searching synsets
	 * @param manual_translate yes if the translate was made manually 
	 */
	public void searchSynset( String word ,boolean manual_translate ) {
		this.manual_translate  = manual_translate;
		translate = word;
		String morphoTag = this.word.getMorphoTag().name();		
		sv.search( word, morphoTag);
	}

	/**
	 * 
	 * Method to annotate the synset to a word
	 * @param index position of selected synset in the list of possible synsets
	 * @param synsets list of possible synsets 
	 * @return yes (was annoted)
	 */
	public boolean note(int index, Synset synsets[]) {
		Ontology g = Ontology.getInstance();
		String options[] = {"Sim", "Não"};
		
		if(!tv.getCheckSubstantivo().isSelected() && !tv.getCheckVerbo().isSelected())
			return false;
		
		if(tv.getCheckSubstantivo().isSelected() && tv.getCheckVerbo().isSelected()) {
			if(word.getMorphoTag() != MorphoTags.NOUN && word.getMorphoTag() != MorphoTags.VERB)
				return false;
		} else if(tv.getCheckSubstantivo().isSelected()) {
			if(word.getMorphoTag() != MorphoTags.NOUN)
				return false;	
		} else if(tv.getCheckVerbo().isSelected()) { 
			if(word.getMorphoTag() != MorphoTags.VERB)
				return false;
		} 
		
		int op = JOptionPane.showOptionDialog( tv,
				"Palavra: " + word.getText() +"\n" +
				"Tradução: " + translate+"\n" +
				"Synset:\n\t " + SynsetCellRender.formatSynonyms( synsets[  index ] )+
						"\n\t" + synsets[ index ].getDefinition(),
				"Deseja realizar esta anotação?",				
				JOptionPane.YES_NO_OPTION,
				JOptionPane.QUESTION_MESSAGE,
				null, 
				options,
				options[ 0 ] );

		if ( op == JOptionPane.YES_OPTION ){			
			
			
			String translate_set[] = t.getTranslates();
			
			LinkedList<Synset> synset_set = new LinkedList<Synset>();
			for( int i=0; i < translate_set.length; i++ ){
				Synset aux[] = WordNet.getSynonymsBySynsetType( translate_set[ i ], this.word.getMorphoTag().name()); // 11-10-13
				if ( aux.length == 0 )
					continue;
				
				for( int j=0; j < aux.length; j++ )
					if ( !synset_set.contains( aux[ j ] ) )
						synset_set.add( aux[ j ]);
			}
			
			if( word.isAnnoted() && word.isAnnotedState()) {
				Node noForDeleting = new Node(SynsetCellRender.formatSynonyms(word.getSynset()), word.getSynset().hashCode());
				g.deleteNode(noForDeleting);
			}
			
			word.setAnnotation( manual_translate, translate, synsets[ index ], translate_set, synset_set );

			WordNet.addTreeForSynset(synsets[index], null);
			t.init();
			sv.init();

			count = (count + 1) % 2;
			if ( count == 0 ) {
				saveAnnotation( false );
				if(name_file.trim().equals("")) {
					Text texts[] = tv.getTexts();
					name_file = texts[ 0 ].getDirectory().getAbsoluteFile() + File.separator + texts[ 0 ].getDirectory().getName();
				}
					 
				
				g.saveAll(new File(name_file));
			}
			isUpdated = true;
		}
		return true;
	}

	/**
	 * This method save the annotation in xml format
	 * @param print_name yes (print a Message Dialog in the UI)
	 */
	public void saveAnnotation( boolean print_name ) {
		
		//Cria uma thread para não parar a anotação
		new Thread("save annotation tmp"){
			@Override
			public void run() {
				try {
	
					Document document = new Document();
					Element root = new Element("save");
					document.setRootElement( root );

					//saving the name of annotators
					Element annoters = new Element("Anotadores");
					int count = 1;
					for( String name : av.getAnnoters() ){
						Element new_annoter = new Element("Anotador");
						new_annoter.setAttribute("id",  "" + count );
						new_annoter.setText( name );
						
						annoters.addContent( new_annoter );
						count++;
					}
					root.addContent( annoters );
					
					count = 1;
					
					Element limitAnnotations = new Element( "LimitesAnotacao" );
					for (String limitPOS : lav.getLimits().keySet()) {
						Element limit_pos = new Element( "Pos" );
						limit_pos.setAttribute( "id", limitPOS );
						limit_pos.setText( ""+lav.getLimits().get(limitPOS) );
						limitAnnotations.addContent( limit_pos );
					}
					root.addContent(limitAnnotations);
					
					Element annoters_files = new Element("Arquivos");
					root.addContent( annoters_files );
					
					Text texts[] = tv.getTexts();
										
					for( Text text : texts ){
						
						Element element_file = new Element("Texto");
						//Write the name file extracting the extension of mxpost
						element_file.setAttribute("name", text.getNameFile().replaceFirst("_mxpost.", ".") );
						element_file.setAttribute("language", text.getLanguage().toString() );		
						annoters_files.addContent( element_file );
						
						count = 0;
						Element paragraph = new Element("p");
						paragraph.setAttribute("number", "" + count );
						element_file.addContent( paragraph );
						
						for( Tokens t : text.getTokens() ){
							
							if( t == null ){																
								count++;
								paragraph = new Element("p");
								paragraph.setAttribute("number", "" + count);
								element_file.addContent( paragraph );
								
							}else{
								
								//Criando o token
								Element element_token = new Element("Token");
								
								Element word = new Element("Valor");
								word.setText( t.getText() );

								word.setAttribute("cp",t.getPartOfComplexPredicate() );
								
								element_token.addContent( word );						
								
												
								Element tag = new Element("Tag");
								tag.setText( t.getTag() );
								element_token.addContent( tag );
								
								Element morpho_tag = new Element("MorphoTag");
								morpho_tag.setText( t.getMorphoTag().toString() );
								element_token.addContent( morpho_tag );
								
								Element morpho_tag_manual = new Element("MorphoTagPOS");
								morpho_tag_manual.setText( t.getMorphoTagPOS().toString() );
								element_token.addContent( morpho_tag_manual );
								
								Element lemma = new Element("Lema");
								lemma.setText( t.getLemma() );
								element_token.addContent( lemma );
								
								Element comment = new Element("Comentario");
								comment.setAttribute("content", t.getComment().name());
								comment.setAttribute("obs", t.getCommentObs());								
																
								element_token.addContent( comment );
															
								Element type = new Element("Type");
								type.setText(""+ t.getAnnotedState() );
								element_token.addContent( type );
								
								Element translates = new Element("Traducoes");																		
								translates.setAttribute("traducao_manual",  (t.getmanual_translate()? "true": "false") );
								//Pegando as traduções
								if( t.getTranslate() != null ){
											
									String translate_set[] = t.getTtranslateSet();								
									for( int i=0; i < translate_set.length; i++){
										
										Element t_element = new Element("Traducao");
										t_element.setText( translate_set[ i ] );
										t_element.setAttribute(
												"selecionado",
												t.getTranslate().compareTo( translate_set[ i ] ) == 0? "true" : "false" );
										
										translates.addContent( t_element );
									}
																			
								}								
								element_token.addContent( translates );
								
								
								Element synsets= new Element("Synsets");								
								Synset s = t.getSynset();
								if ( s != null ){
									LinkedList<Synset> synset_set = t.getSynsetSet();									
									for( int i=0; i < synset_set.size(); i++ ){
										Element synset_element = new Element("Synset");
										synset_element.setText(""+ synset_set.get( i ).hashCode() );
										synset_element.setAttribute(												
												"selecionado",
												synset_set.get( i ).hashCode() == t.getSynset().hashCode() ? "true" : "false" );
										
										synsets.addContent( synset_element );
										
									}
								}																				
								element_token.addContent( synsets );
								
								paragraph.addContent( element_token );
							}
						}
					}
					
					File p = new File(					
							texts[ 0 ].getDirectory().getAbsoluteFile() +
							File.separator +   
							texts[ 0 ].getDirectory().getName() +
							".xml");
					
					name_file = texts[ 0 ].getDirectory().getAbsoluteFile() +
							File.separator +   
							texts[ 0 ].getDirectory().getName(); 
					XMLOutputter out = new XMLOutputter();					
					out.output( document, new FileOutputStream( p ));
					System.out.println("Save tmp successful!");
					
				} catch (IOException e) {			
					e.printStackTrace();
				}
			}
		}.start();
		
		if( print_name )
			JOptionPane.showMessageDialog( tv, 
				"Arquivo xml da anotação salvo:\n"+
				tv.getTexts()[ 0 ].getDirectory().getName() +".xml", 
				"Salvando anotação!",
				JOptionPane.INFORMATION_MESSAGE );
		
	}

	/**
	 * Method for removing the annotation of a word
	 */
	public void removeAnnotation() {		
		if ( word != null ){
			Ontology g = Ontology.getInstance();
			Node noForDeleting = new Node(SynsetCellRender.formatSynonyms(word.getSynset()), word.getSynset().hashCode());
			g.deleteNode(noForDeleting);
			word.removeAnnotation();
			t.init();
			sv.init();
			isUpdated = true;
		}
	}
	
	public void setTranslateLabel( String label, int index  ){		
		t.setTranslateLabel( label , index );
	}

	public void setSelectedSynset(int index) {
		sv.setSelectedSynset( index );
	}

	public String getName_file() {
		return name_file;
	}

	public void setName_file(String name_file) {
		this.name_file = name_file;
	}

	/**
	 *  This method enable the CommentView for a word
	 */
	public void enableComment() {
		
		if( word.getComment().equals(Comment.NO_COMMENTS) ) { 
			cv.enableTextComment();	
			cv.enableButtonComment();
			if( word.getPartOfComplexPredicate().startsWith("CP") )
				cv.setComboComment(1);
			else
				cv.setComboComment(0);
		} else {
			cv.disableTextComment();
			cv.disableButtonComment();
			cv.setComment( word.getComment(), word.getCommentObs() );
		}
			
	}
	
	/**
	 * This method save the comment written by a annotator
	 * @param comment content of comment
	 */
	public void setComment(Comment comment) {
		Ontology g = Ontology.getInstance();
		word.setComment(comment);
		if( word.getComment().equals(model.Comment.AUXILIAR_VERB) ) {
			if( word.isAnnoted() && word.isAnnotedState()) {
				Node noForDeleting = new Node(SynsetCellRender.formatSynonyms(word.getSynset()), word.getSynset().hashCode());
				g.deleteNode(noForDeleting);
			}
			word.setCommentAnnotation();
			t.init();
			sv.clear();
		} else if(word.getComment().equals(model.Comment.ANNOTATION_ERROR) ){
			if( word.isAnnoted() && word.isAnnotedState()) {
				Node noForDeleting = new Node(SynsetCellRender.formatSynonyms(word.getSynset()), word.getSynset().hashCode());
				g.deleteNode(noForDeleting);
			}
			word.setCommentAnnotation();
		} 
		isUpdated = true;
	}
	
	public Comment getComment() {
		return word.getComment();
	}
	
	public void setCommentObs(String commentObs) {
		word.setCommentObs(commentObs);
	}
	
	public Map<String,Double> getLimitsAnnotation() {
		return lav.getLimits();
	}

	public void setMorphoTagtoToken(MorphoTags morphoTag) {
		word.setMorphoTag(morphoTag);		
	}
	
	public void changeType ( StateAnnotation stateAnnotation ) {
		word.changeType( stateAnnotation );	
		word.activeBorder();
	}

	public void modifyAnnotationAllowed() {
		tv.makeFrequency();		
		tv.modifyCheckboxs();
	}

	public void setTag(String tag) {
		word.setTag( tag );
		try {
			word.setLemma( Lemmar.lemma(word.getLemma(), tag));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	
	
}