package model;

/**
 * Enum that represents the tags 
 * @author gonta
 *
 */
public enum MorphoTags {
	NONE("Nenhuma etiquetada"), 

	NOUN("Substantivos"),

	VERB("Verbos"),

	VAUX("Verbo Auxiliar"),

	ADVERB("Advérbio"),

	ADJECTIVE("Adjetivo"),
	
	//Symbols, numbers, others
	OTHER("Outra");
	
	private String description;	
	
	
	MorphoTags( String description ){
		this.description = description;
	}
	
	/**
	 * Convert the value showed for the user to a tag value
	 * 
	 * @param str value sended by the user
	 */
	public static MorphoTags strToTag( String str ){
		
		if( str.compareToIgnoreCase( "Nenhuma etiquetada") == 0 )
			return NONE;
		
		if( str.compareToIgnoreCase( "Substantivos") == 0 )
			return NOUN;
		
		if( str.compareToIgnoreCase( "Verbos") == 0 )
			return VERB;
		
		if( str.compareToIgnoreCase( "Verbo Auxiliar") == 0 )
			return VAUX;
		
		if( str.compareToIgnoreCase( "Advérbio") == 0 )
			return ADVERB;
		
		if( str.compareToIgnoreCase( "Adjetivo") == 0 )
			return ADJECTIVE;
		
		if( str.compareToIgnoreCase( "Outra") == 0 )
			return OTHER;
		
		return null;
	}
	
	/**
	 * Mapping a mxpost tag to a enum
	 * 
	 * @param tag A string representing a mxpost tag 
	 * @return a enum found 
	 */
	public static MorphoTags mxpostMap( String tag ){
	
		if ( tag.compareTo("N") == 0) return MorphoTags.NOUN;
	
		if ( tag.compareTo("VERB") == 0 ||  tag.compareTo("V") == 0 ) return	MorphoTags.VERB;
		
		if ( tag.compareTo("VAUX") == 0 ||  tag.compareTo("AUX") == 0 ) return	MorphoTags.VAUX;
	
		if ( tag.compareTo("ADJ") == 0 ) return MorphoTags.ADJECTIVE;
	
		if ( tag.compareTo("ADV") == 0 ) return MorphoTags.ADVERB;		
	
		if( !Character.isLetterOrDigit( tag.charAt( 0 ) ) || tag.compareTo("NUM") == 0)
			return MorphoTags.OTHER;
	
		return NONE;
	}
	
	/**
	 *  Mapping a TreeTagger tag to a enum
	 * 
	 * @param tag A string representing a TreeTagger tag 
	 * @return a enum found
	 */
	public static MorphoTags treetaggerMap( String tag ){
		
		/**
		 * TAGS of english found in 
		 * :<a href='http://www.ims.uni-stuttgart.de/projekte/corplex/TreeTagger/Penn-Treebank-Tagset.pdf'>http://www.ims.uni-stuttgart.de/projekte/corplex/TreeTagger/Penn-Treebank-Tagset.pdf</a>
		 * */		
		
		if( tag.startsWith("NN") ) return NOUN;
		
		if( tag.startsWith("VP") ) return VERB;
		
		if ( tag.startsWith("JJ") ) return ADJECTIVE;
		
		if ( tag.startsWith("RB")) return ADVERB;
		
		//Símbolos
		if ( tag.compareTo("SYM") == 0 || tag.compareTo("QT")==0 ||
				tag.compareTo("DOTS") == 0 || tag.compareTo("COLON") == 0 || 
				tag.compareTo("CM") == 0 || tag.compareTo("BACKSLASH")==0 ) return OTHER;
		
		/**
		 * TAGS of spanish found in
		 * :<a href='ftp://ftp.ims.uni-stuttgart.de/pub/corpora/spanish-tagset.txt'>ftp://ftp.ims.uni-stuttgart.de/pub/corpora/spanish-tagset.txt</a>
		 * 
		 */
		if( tag.startsWith("NM") ) return NOUN;
		
		if( tag.startsWith("V") ) return VERB;
		
		if ( tag.compareTo("ADJ") == 0) return ADJECTIVE;
		
		if ( tag.compareTo("ADV") == 0 ) return ADVERB;
		
		return NONE;
	}
	
	
	@Override
	public String toString() {
		return this.description;
	}
}
