package view;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

import controller.Annotation;

/**
 * This class represents the comment view component 
 * @author gonta
 *
 */
public class CommentView extends JPanel {
	
	
	private JTextField comment_txtField;
	private JComboBox<String> comment_box;
	private JButton addComment_button;
	
	/**
	 * Constructor
	 * @param jframe
	 */
	public CommentView( JFrame jframe ) {
		super();	
		setBorder( new TitledBorder("Comentário")) ;
		addComponents();

	}

	private void addComponents() {
	
		comment_txtField = new JTextField();
		comment_txtField.setSize(100, 35);
		comment_txtField.setEnabled(false);
		comment_txtField.setToolTipText("Clique duas vezes para fazer modificações");
		comment_txtField.addMouseListener(new MouseListener() {
			
			@Override
			public void mouseReleased(MouseEvent arg0) {}
			
			@Override
			public void mousePressed(MouseEvent arg0) {}
			
			@Override
			public void mouseExited(MouseEvent arg0) {}
			
			@Override
			public void mouseEntered(MouseEvent arg0) {}
			
			@Override
			public void mouseClicked(MouseEvent evt) {
				if( evt.getClickCount() == 2 ) {
					comment_txtField.setEnabled( true );
					addComment_button.setEnabled( true );
					comment_box.setEnabled( true );
				}
				
			}
		});
		
		addComment_button = new JButton();
		addComment_button.setText("Adicionar");
		addComment_button.setEnabled(false);
		
		comment_box = new JComboBox<String>();
		comment_box.setToolTipText("Clique duas vezes para fazer modificações");
		
		for (model.Comment comment : model.Comment.values()) {
			comment_box.addItem(comment.toString());
		}
		
		comment_box.setEnabled(false);
		comment_box.addMouseListener(new MouseListener() {
			
			@Override
			public void mouseReleased(MouseEvent evt) {}
			
			@Override
			public void mousePressed(MouseEvent evt) {}
			
			@Override
			public void mouseExited(MouseEvent evt) {}
			
			@Override
			public void mouseEntered(MouseEvent evt) {}
			
			@Override
			public void mouseClicked(MouseEvent evt) {
				if( evt.getClickCount() == 2 ) {
					comment_box.setEnabled( true );
					comment_txtField.setEnabled( true );
					addComment_button.setEnabled( true );
				}
			}
		});
		
		JPanel p2 = new JPanel();
		
		p2.setBorder( BorderFactory.createEmptyBorder(10, 5, 10, 5)); 
		p2.setLayout( new BoxLayout( p2, BoxLayout.X_AXIS ));
	
		JPanel p3 = new JPanel();
		
		p3.setBorder( BorderFactory.createEmptyBorder(10, 5, 10, 5)); 
		p3.setLayout( new BoxLayout( p3, BoxLayout.Y_AXIS ));
	
		p2.add( comment_box );
		p2.add( Box.createRigidArea(new Dimension(20,0)) );
		p2.add( addComment_button );
		p3.add( p2 );
		p3.add( comment_txtField );		
		
		setLayout( new BorderLayout() );
		add( p3 , BorderLayout.PAGE_END);
		
	}
	
	/**
	 * Enable the comment textfield
	 */
	public void enableTextComment() {
		comment_txtField.setEnabled(true);
		comment_box.setEnabled(true);
		comment_txtField.setText("");
	}

	/**
	 * Disable the comment textfield
	 */
	public void disableTextComment() {
		comment_txtField.setEnabled(false);
		comment_box.setEnabled(false);
	}
	
	/**
	 * Enable the comment button
	 */
	public void enableButtonComment() {
		addComment_button.setEnabled(true);
	}
	
	/**
	 * Disable the comment button
	 */
	public void disableButtonComment() {
		addComment_button.setEnabled(false);
	}

	/**
	 * This method set the controller to the class for handling
	 * the read/write of the annotation
	 * @param annotation Annotation of a word
	 */
	public void setController(final Annotation annotation) {
		
		final Component component = this;
		addComment_button.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent e) {
				try {					
						annotation.setCommentObs(comment_txtField.getText());						
						annotation.setComment(model.Comment.getComment(String.valueOf(comment_box.getSelectedItem())));				
						comment_txtField.setEnabled(false);
						comment_box.setEnabled(false);
						addComment_button.setEnabled(false);
						JOptionPane.showMessageDialog(component, "Adicionado com sucesso!");						
				} catch( Exception ex ) {
					ex.printStackTrace();
					JOptionPane.showMessageDialog(component, "Erro ao adicionar comentário");
				}				
			}
		});
		
	}

	/**
	 * This method set the value of the comment
	 * @param comment comment content
	 * @param commentObs observation of the comment
	 */
	public void setComment(model.Comment comment, String commentObs) {
		comment_txtField.setText(commentObs);
		comment_box.setSelectedIndex(0);
		for (int i = 0; i < comment_box.getItemCount(); i++) {
			String selectedItem = comment_box.getItemAt(i);
			if ( selectedItem.equals( comment.toString() ) ) {
				comment_box.setSelectedIndex(i);
				break;
			}				
		}
	}

	/**
	 * This method set the comment selected by the user
	 * @param i index selected
	 */
	public void setComboComment(int i) {
		comment_box.setSelectedIndex(i);
	}
	
	/**
	 * This method set observation comment 
	 * @param obs observation comment
	 */
	public void setTextComment(String obs) {
		comment_txtField.setText(obs);
	}

}
